"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.convertDirection = convertDirection;
exports.convertDistanceUnit = convertDistanceUnit;
exports.convertPrepend = convertPrepend;
exports.convertUnitName = convertUnitName;
exports.generateInstructionText = generateInstructionText;
exports.getCurrentInstruction = getCurrentInstruction;
exports.getDistanceAlongPath = getDistanceAlongPath;
exports.getDistanceBetweenPoints = getDistanceBetweenPoints;
exports.getPerpendicularDistanceFromPath = getPerpendicularDistanceFromPath;
exports.getSurroundingPoints = getSurroundingPoints;
exports.preProcessPath = void 0;
const METERS_PER_PIXEL = 0.05962332392785686;
function getCurrentInstruction(userLocation, instructions) {
  var _closestInstruction;
  let dist_unit = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'METERS';
  let locale = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 'en-US';
  var closestInstruction = undefined;
  instructions.forEach(function (instruction) {
    const distance = getPerpendicularDistanceFromPath(userLocation, instruction.startPoint, instruction.endPoint);
    const distanceAlongPath = getDistanceAlongPath(userLocation, instruction.startPoint, instruction.endPoint);

    // Ignore matches if not sufficiently within bounds of path
    if (!(distanceAlongPath > 0 && distanceAlongPath < 1)) {
      return;
    }
    let remainingDistance = getDistanceBetweenPoints(userLocation, instruction.endPoint);
    remainingDistance = Math.round(remainingDistance * METERS_PER_PIXEL * 100) / 100;
    const fullText = generateInstructionText({
      ...instruction,
      remainingDistance
    }, dist_unit, locale);
    if (closestInstruction === undefined) {
      closestInstruction = {
        instruction: {
          ...instruction,
          remainingDistance,
          fullText
        },
        distance
      };
    } else {
      if (distance < closestInstruction.distance) {
        closestInstruction = {
          instruction: {
            ...instruction,
            remainingDistance,
            fullText
          },
          distance
        };
      }
    }
  });
  // @ts-ignore
  return (_closestInstruction = closestInstruction) === null || _closestInstruction === void 0 ? void 0 : _closestInstruction.instruction;
}
function getSurroundingPoints(path, instructions, waypointId) {
  let instructionIndex = 0;
  let leadWaypointIndex = -1;
  for (let i = 0; i < path.points.length; i++) {
    var _path$points$i, _path$points$i2, _instructions$instruc;
    if (path.points[i] && ((_path$points$i = path.points[i]) === null || _path$points$i === void 0 ? void 0 : _path$points$i.id) == waypointId) {
      leadWaypointIndex = instructionIndex;
      break;
    }
    if (instructions[instructionIndex] && ((_path$points$i2 = path.points[i]) === null || _path$points$i2 === void 0 ? void 0 : _path$points$i2.id) == ((_instructions$instruc = instructions[instructionIndex]) === null || _instructions$instruc === void 0 ? void 0 : _instructions$instruc.wp)) {
      instructionIndex += 1;
    }
  }
  return leadWaypointIndex;
}
const preProcessPath = (path, instructions) => {
  var pathSegmentIndex = 0;
  var prevPathIndex = 0;
  var prevInstructionIndex;
  console.log('Path Length', JSON.stringify(path));
  console.log('Instruction Length' + instructions.length);
  path.forEach(function (floorPath) {
    floorPath.points.forEach(function (pathSegment) {
      var _pathSegment$associat, _pathSegment$associat2;
      console.log('reviewing path segment: ', (_pathSegment$associat = pathSegment.associations) === null || _pathSegment$associat === void 0 ? void 0 : _pathSegment$associat.amenities);
      if (pathSegment.associations && (_pathSegment$associat2 = pathSegment.associations) !== null && _pathSegment$associat2 !== void 0 && _pathSegment$associat2.amenities) {
        var _pathSegment$associat3;
        let foundDoor = false;

        // Check if this waypoint is a door
        (_pathSegment$associat3 = pathSegment.associations) === null || _pathSegment$associat3 === void 0 || _pathSegment$associat3.amenities.forEach(function (amenity) {
          if (amenity == 13237) {
            foundDoor = true;
          }
        });

        // if the waypoint has a door find, inject a new nav instruction into the path.
        if (foundDoor) {
          var _instructions$index, _instructions$index2;
          console.log('Path Segment ID' + pathSegment.id);
          const index = getSurroundingPoints(floorPath, instructions, pathSegment.id);
          const splitInstruction = {
            fullText: 'Proceed through the Door',
            prepend: '',
            direction: 'Proceed through the Door',
            distance: ((_instructions$index = instructions[index]) === null || _instructions$index === void 0 ? void 0 : _instructions$index.distance) || 1,
            remainingDistance: ((_instructions$index2 = instructions[index]) === null || _instructions$index2 === void 0 ? void 0 : _instructions$index2.remainingDistance) || 1,
            wp: pathSegment.id,
            dest: -1,
            startPoint: [],
            endPoint: []
          };
          instructions.splice(index, 0, splitInstruction);
        }
      }
    });
  });
  instructions.forEach(function (instruction, instructionIndex) {
    for (var i = pathSegmentIndex; i < path.length; i++) {
      var _path$i;
      const pathPoints = (_path$i = path[i]) === null || _path$i === void 0 ? void 0 : _path$i.points;
      if (pathPoints === undefined) {
        continue;
      }
      for (var j = prevPathIndex; j < pathPoints.length; j++) {
        const point = pathPoints[j];
        if (point === undefined) {
          continue;
        }
        if (point.id === instruction.wp) {
          instruction.endPoint = [point.x, point.y];
          if (prevInstructionIndex !== undefined) {
            instruction.startPoint = instructions[prevInstructionIndex].endPoint;
          } else {
            var _pathPoints$, _pathPoints$2;
            instruction.startPoint = [(_pathPoints$ = pathPoints[0]) === null || _pathPoints$ === void 0 ? void 0 : _pathPoints$.x, (_pathPoints$2 = pathPoints[0]) === null || _pathPoints$2 === void 0 ? void 0 : _pathPoints$2.y];
          }
          prevInstructionIndex = instructionIndex;
          return;
        }
        prevPathIndex = j;
      }
      pathSegmentIndex = i;
      prevPathIndex = 0;
    }
  });
  return instructions;
};

// get distance along path as proportion of total path length
exports.preProcessPath = preProcessPath;
function getDistanceAlongPath(point, startPathPoint, endPathPoint) {
  const x1 = startPathPoint[0];
  const y1 = startPathPoint[1];
  const x2 = endPathPoint[0];
  const y2 = endPathPoint[1];
  const x0 = point[0];
  const y0 = point[1];
  const numerator = (x2 - x1) * (x0 - x1) + (y2 - y1) * (y0 - y1); // dot product
  const denominator = Math.pow(y2 - y1, 2) + Math.pow(x2 - x1, 2);
  return numerator / denominator;
}

// Get perpendicular distance from point to path
function getPerpendicularDistanceFromPath(point, startPathPoint, endPathPoint) {
  const x1 = startPathPoint[0];
  const y1 = startPathPoint[1];
  const x2 = endPathPoint[0];
  const y2 = endPathPoint[1];
  const x0 = point[0];
  const y0 = point[1];
  const numerator = Math.abs((y2 - y1) * x0 - (x2 - x1) * y0 + x2 * y1 - y2 * x1);
  const denominator = Math.sqrt(Math.pow(y2 - y1, 2) + Math.pow(x2 - x1, 2));
  return numerator / denominator;
}
function getDistanceBetweenPoints(point1, point2) {
  const x1 = point1[0];
  const y1 = point1[1];
  const x2 = point2[0];
  const y2 = point2[1];
  return Math.sqrt(Math.pow(x1 - x2, 2) + Math.pow(y1 - y2, 2));
}
function generateInstructionText(instruction, dist_unit, locale) {
  const distance = Math.round(convertDistanceUnit(instruction.remainingDistance, dist_unit));
  const prepend = convertPrepend(instruction.prepend, locale);
  const direction = convertDirection(instruction.direction, locale);
  const atTranslated = locale === 'es-ES' ? 'en' : 'at';
  const destinationName = instruction.destinationName;
  const inTranslated = locale === 'es-ES' ? 'en' : 'in';
  const unitName = convertUnitName(dist_unit, locale, distance != 1);
  if (destinationName) return `${prepend}${direction} ${atTranslated} ${destinationName} ${inTranslated} ${distance} ${unitName}`;else return `${prepend}${direction} ${inTranslated} ${distance} ${unitName}`;
}
function convertDistanceUnit(distance, dist_unit) {
  switch (dist_unit) {
    case 'METERS':
      return distance;
    case 'FEET':
      return distance * 3.28084;
    case 'INCHES':
      return distance * 39.3701;
    default:
      return distance;
  }
}
function convertPrepend(prepend, locale) {
  switch (prepend) {
    case 'Turn ':
      switch (locale) {
        case 'es-ES':
          return 'Gire ';
        default:
          return prepend;
      }
    case 'Arrive ':
      switch (locale) {
        case 'es-ES':
          return 'Llegue ';
        default:
          return prepend;
      }
    default:
      return prepend;
  }
}
function convertDirection(prepend, locale) {
  switch (prepend) {
    case 'Left':
      switch (locale) {
        case 'es-ES':
          return 'Izquierdo';
        default:
          return prepend;
      }
    case 'Right':
      switch (locale) {
        case 'es-ES':
          return 'Derecho';
        default:
          return prepend;
      }
    case 'Forward':
      switch (locale) {
        case 'es-ES':
          return 'hacia adelante';
        default:
          return prepend;
      }
    default:
      return prepend;
  }
}
function convertUnitName(unit, locale, plural) {
  switch (unit) {
    case 'METERS':
      switch (locale) {
        case 'es-ES':
          return plural ? 'metros' : 'metro';
        default:
          return plural ? 'meters' : 'meter';
      }
    case 'FEET':
      switch (locale) {
        case 'es-ES':
          return plural ? 'pies' : 'pie';
        default:
          return plural ? 'feet' : 'foot';
      }
    case 'INCHES':
      switch (locale) {
        case 'es-ES':
          return plural ? 'pulgada' : 'pulgadas';
        default:
          return plural ? 'inches' : 'inch';
      }
    default:
      return unit;
  }
}
//# sourceMappingURL=Instructions.js.map